"use strict";
this.name        = "deep_space_dredger"; 
this.author      = "eric walch"; 
this.copyright   = "� 2008 Walch & Sabre & Little Bear"; 
this.description = "This script places the dredgers in deep space."; 
this.version     = "2.4.6"; 


this.startUp = function ()
{
    this.dredger = true
    if (this.dredgerTimer)
        this.dredgerTimer.start()
    else
        this.dredgerTimer = new Timer(this, this.dredgerCheck, 150, 120);
}

this.shipExitedWitchspace = function ()
{
    if (this.salvageList) {delete this.salvageList; delete this.salvageNameList};
    if (system.isInterstellarSpace || system.sun.hasGoneNova) return;
	this.dredger = false;
    if (system.techLevel > 8)
	{
		if (Math.random() < 0.5)
		{
			if (Math.random() < 0.2)
            {
                system.addShipsToRoute("dredgertrader", 1, Math.random(), "pw");
            }
            this.dredger = true; // dredger can be added to this system.
		}
		else
		{
			if (Math.random() < 0.05 && !worldScripts["bigShips_populator"]) 
            {
                system.addShipsToRoute("dredgers", 1, Math.random()*0.5, "pw");
            }
		}
	}
    
	if (system.government < Math.random() * 7) // more change in low governments
	{
		if (Math.random() < 0.1) this.addDerelict(0.1+Math.random()*0.8, 1);
		if (Math.random() < 0.02)
		{
			var location = Math.ceil(Math.random()*5);
			switch(location)
			{
			case 1:     { 
                            this.addDerelict(0.97, 5);  // 20%
                            player.commsMessage(expandDescription("[salvage_drone_battle_station]"), 6); 
                            break;
                        }
			case 5:     {
                            this.addDerelict(0, 5);  // 20%
                            player.commsMessage(expandDescription("[salvage_drone_battle_witchpoint]"), 6); 
                            break;
                        }
			default:    {
                            this.addDerelict(0.2+Math.random()*0.7, 5);  // 60%
                            player.commsMessage(expandDescription("[salvage_drone_battle_spacelane]"), 6); 
                            break;
                        }
			}
			this.dredger = true;
		}
	}
}

this.addDerelict = function (location, count)
{
	if (!system.mainStation) return;
    for (var j = 0; j < count; j++)
    {
        var derelict = system.addShipsToRoute("trader", 1, location)[0];
        derelict.bounty = 0;
        
        // set script to default, to avoid a special script for the trader doing stuff. (like setting a new AI)
        if (derelict.script.name != "oolite-default-ship-script") derelict.setScript("oolite-default-ship-script.js");
        derelict.switchAI("nullAI.plist");

        if (derelict.escorts)
        {
            for (var i = 0; i < derelict.escorts.length; i++) 
            {
                derelict.escorts[i].remove(true);
            }
        }
        
        if (derelict.mass > 1E6) // those probably can't dock so we don't want them.
        {
            derelict.remove(true);
        } 
        else
        {
            derelict.script.shipLaunchedEscapePod = this.$shipLaunchedEscapePod; // function to remove the escapepod after launch.
            if (derelict.equipmentStatus("EQ_ESCAPE_POD") == "EQUIPMENT_UNAVAILABLE") derelict.awardEquipment("EQ_ESCAPE_POD")
            derelict.abandonShip(); // make sure no pilot is left behind and this command turns the ship into cargo.
            derelict.primaryRole = "dredgerDerelict"; // to avoid pirate attacks
            derelict.displayName = derelict.displayName + " (derelict)";
        }
    }
}

this.dredgerCheck = function ()
{
    if (this.dredger && player.ship.speed > player.ship.maxSpeed && !system.isInterstellarSpace && player.ship.position.distanceTo(system.mainPlanet.position) > 1.5E6) 
    { 
        this.addDredger(); 
        if(Math.random()< 0.5) this.dredger = false;
    }
}

this.addDredger = function ()
{
    var orientation = player.ship.orientation; 
    var forwardPosition = player.ship.position.add(orientation.vectorForward().multiply(300E3));
    system.addShips("dredgers", 1, forwardPosition, 1);
    if (Math.random()<0.5) 
    {
        system.addShips("asteroid", Math.ceil(Math.random()*3), forwardPosition, 10E3);
        system.addShips("miner", Math.ceil(Math.random()*3), forwardPosition, 10E3);
    }
}

// Below are functions used by ships
this.checkSalvage = function (ship)
{
    if (!this.salvageList) return false;
    if (this.salvageList.indexOf(ship) > -1) return true;
    return false;
}

this.addSalvage = function (ship)
{
    if (!this.salvageList)
    {
        this.salvageList = [ship];
        this.salvageNameList = [ship.displayName];
        return true;
    }
    if (this.salvageList.indexOf(ship) > -1) return false;
    this.salvageList.push(ship)
    this.salvageNameList.push(ship.displayName);
    return true;
}

this.removeSalvage = function (ship)
{
    if (!this.salvageList) return false;
    var i = this.salvageList.indexOf(ship)
    if (i> -1)
    {
        this.salvageList.splice(i,1)
        this.salvageNameList.splice(i,1)
        return true;
    }
    return false;
}

this.startScanner = function (salvager) // activated by dredgerscript
{
	this.salvageShip = salvager;
	this.timerCount = 0;
    if (this.salvageTimer)
        this.salvageTimer.start()
    else
        this.salvageTimer = new Timer(this, this.scanSalvage, 15, 15)
}

this.scanSalvage= function ()
{
    if (!this.salvageList || this.salvageList.length === 0 || !this.salvageShip.isValid) this.salvageTimer.stop()
    for (var i=0; i<this.salvageList.length;i++)
    {
        if (!this.salvageList[i].isValid)  // if ship died.
        {
            this.salvageList.splice(i,1)
            var lostShipName = this.salvageNameList.splice(i,1)
            player.commsMessage(expandDescription("[salvage_drone_intro] ") + this.salvageShip.displayName + expandDescription(" [salvage_drone_killed] ") + lostShipName, 6);
			return;
        }
    }
	if (this.timerCount++ > 10 && this.salvageNameList.length > 0)
	{
		player.consoleMessage(expandDescription("[salvage_drone_intro] ") + this.salvageShip.displayName + " is still waiting for incoming " + this.salvageNameList[0], 6);
		this.timerCount = 0;
	}
}

// next function is attached to derelict ships
this.$shipLaunchedEscapePod = function (pod)
{
    pod.remove(true); // we don't want floating escapepods around but need them initially to create the derelict.
}

// Below are scripts attached by missile-script to the derelict ships
this.derelictDied = function (ship)
{
	var cargo = Math.ceil(Math.random()*10);
    if (ship.cargoSpaceAvailable < cargo) cargo = ship.cargoSpaceAvailable;
	if (cargo > 0) ship.spawn("cargopod", (Math.random() * 5 + 5));
}

this.derelictDocked = function (station)
{
    if (worldScripts.deep_space_dredger.checkSalvage(this.ship))
    {
        /*
        // Suggestion by Switek
		var cargo = this.ship[maxCargo];
		if (cargo > 20) cargo = Math.round( ((cargo - 20) * Math.random()) + 20 ); // Cargo total is 20 to max tons for big haulers.
		var payment = Math.round(this.ship.mass * 0.01 + cargo * (10 + 50 * Math.random() ) ); // Cargo is randomly valued at 10-60 credits/ton
		if (payment < 5000) payment = payment + 3000; // +3000 = Enough to pay for the misile, making this profitable[/quote]
        */
        var cargo = Math.round(this.ship.cargoSpaceAvailable / 2.5 * Math.random());
        var payment = Math.round(this.ship.mass/1000) * 10 + cargo * 13;
        if (this.ship.bounty > 0) payment *= 1.5;  // give a 50% bonus for criminal ships.
        missionVariables.salvage_drone_award = payment;
        missionVariables.salvage_drone_cargo = cargo;
        missionVariables.salvage_drone_dname = station.displayName;
        missionVariables.salvage_drone_wname = this.ship.name;
        player.commsMessage(expandDescription("[salvage_drone_congrats]"), 6); // should always reach player.
        player.credits += payment;
        missionVariables.salvage_drone_award = null;
        missionVariables.salvage_drone_cargo = null;
        missionVariables.salvage_drone_dname = null;
        missionVariables.salvage_drone_wname = null;
        worldScripts.deep_space_dredger.removeSalvage(this.ship); // remove from the approach list in the main script.
    }
}