"use strict";
this.name      = "dredgerShip"; 
this.author    = "eric walch"; 
this.copyright   = "� 2008 Walch & Sabre & Little Bear"; 
this.description = "Script for dredger ship";
this.version   = "1.3.6";

this.dockedShuttles = 0;
this.doorY = 0;
this.door = this.ship.subEntities[1]; // door must be defines as the second subentity.

this.shipSpawned = function ()
{
    this.shuttles = Math.round(Math.sqrt(this.ship.mass)/2000); // 20 till 80
    this.ship.primaryRole = "dredgers"; // untill Oolite 1.73, we need this primaryRole for the commodity.plist working correctly.
    if (this.ship.displayName === this.ship.name)
        this.ship.displayName = expandDescription("[salvage_drone_gotit11] [salvage_drone_gotit12]");
    this.oldOolite = (0 < oolite.compareVersion("1.75.1")); // since 1.75.1 it became possible to reliably remove timers on ship removal.
    if (!this.oldOolite)
    {
        delete this.shipDied;
        delete this.playerWillEnterWitchspace;
    }
}

this.checkLaunches = function ()
{
    if (this.doorY >= 1 && this.shuttles > 0) // door fully open.
    {
        this.shuttles--;
        this.ship.reactToAIMessage("LAUNCH_SHUTTLE"); 
    }
    else if (!this.waitForDocking)
    {
        this.ship.reactToAIMessage("RESUME_FLIGHT");
        this.shuttles += this.dockedShuttles;
        this.closeDoor();
    }
}

this.dockingRequested = function ()
{
	this.waitForDocking = true;
}

this.dockingComplete = function ()
{
	this.waitForDocking = false;
}

this.launchFromPlanet= function ()
{
	worldScripts.deep_space_dredger.dredgerWaitingShip = this.ship;  // used as target by rising shuttles.
	var launchVector = this.ship.position.subtract(system.mainPlanet.position).direction();
	var launchSite = system.mainPlanet.position.add(launchVector.multiply(system.mainPlanet.radius + 2500));
	var shuttles = system.addShips("dredgershuttle_up", 8, launchSite, 1000);
    
    var launchOrientation = launchVector.rotationTo(new Vector3D(0,0,1)); // align the quaternion to the launchVector
    for (var i=0; i<shuttles.length; i++)
    {
        shuttles[i].orientation = launchOrientation; // now all shuttles should point away from planet.
    }
    
	this.waitForDocking = true;
    this.openDoor();
}

this.otherShipDocked = function (ship)
{
    if (ship.primaryRole == "dredgershuttle_up") this.dockedShuttles++;
}

this.stationLaunchedShip = function (ship)
{
    if (ship.primaryRole == "miner" && this.oldOolite) ship.setAI("dredgerLaunchHelperAI.plist"); // added to make sure ships don't turn to soon.
    if (ship.isPlayer)
    {
        this.doorY = 1;
        this.closeDoor();
        this.door.orientation = [1, 1, 0, 0]; // start with full open door.
    }
}

this.playerWillEnterWitchspace = function ()
{
    this.$cleanupTimers();
};

this.startScanner = function ()
{
	worldScripts.deep_space_dredger.startScanner(this.ship); // transfer this dredgers entity for messages.
}

this.shipDied = function ()
{
    this.$cleanupTimers();
}

this.entityDestroyed = function ()
{
    // handler was introduced in Oolite 1.75.1
    this.$cleanupTimers();
}

this.initiateMinerLaunch = function ()
{
    this.minersToLaunch = true;
}

this.openDoor = function ()
{
    if (this.doorY >= 1) return; // door is open
    this.openSpeed = 0.005;
    this.$startTimers();
}

this.openDoorFast = function ()
{
    if (this.doorY >= 1) return; // door is open
    this.openSpeed = 0.05;
    this.$startTimers();
}

this.closeDoor = function ()
{
    if (this.doorY <= 0) return; // door is closed
    this.openSpeed = -0.005;
    this.$startTimers();
}

this.moveDoor = function (delta)
{
    if (!delta) delta = 0.25;
    if (!this.ship || !this.ship.isValid || !this.ship.status) {this.$cleanupTimers();return;};
    this.doorY += this.openSpeed*4*delta;
    if (this.doorY >= 1) // door is open
    {
        this.$cleanupTimers();
        this.doorY = 1;
        if (this.minersToLaunch)
        {
            this.minersToLaunch = false;
            this.ship.launchMiner();
            this.ship.launchShipWithRole("miner");
        }
    }
    if (this.doorY <= 0)  // door is closed
    {
        this.$cleanupTimers();
        this.doorY = 0;
    }
    this.door.orientation = [1, this.doorY, 0, 0];
};


// function called by Oolite
(function () {
    const escortPositions =
    [
        // |.|-shape escort pattern
        new Vector3D(-2,  1, -1),
        new Vector3D( 2,  1, -1),
        new Vector3D(-2, -1, -1),
        new Vector3D( 2, -1, -1),
        new Vector3D(-2,  0, -1),
        new Vector3D( 2,  0, -1),
        new Vector3D(-1,  0, -2),
        new Vector3D( 1,  0, -2)
    ];

    this.coordinatesForEscortPosition = function (index) {
        var highPart = 1 + (index >> 3) / 3;
        var lowPart = index % 6;
        
        var spacing = this.ship.collisionRadius * highPart;
        
        return escortPositions[lowPart].multiply(spacing);
    };
}).call(this);


this.$startTimers = function ()
{
    if(this.oldOolite) 
    {
        if (this.moveTimer) this.moveTimer.start()
        else this.moveTimer = new Timer(this, this.moveDoor, 15, 0.25);
    }
    else
    {
        if (!isValidFrameCallback(this.$callbackID)) this.$callbackID = addFrameCallback(this.moveDoor.bind(this));
    }
}

this.$cleanupTimers = function ()
{
    if (this.moveTimer) {this.moveTimer.stop(); delete this.moveTimer} // for oolite 1.74
	if (this.$callbackID && isValidFrameCallback(this.$callbackID)) // for Oolite 1.75+
	{ 
		removeFrameCallback(this.$callbackID);
        delete this.$callbackID;
	}
}